/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.java.net
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.util.test;

import javax.annotation.Nonnull;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import org.openide.util.Lookup;
import org.openide.util.LookupListener;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;
import static org.junit.Assert.*;

/**
 * Mock implementation of system default lookup suitable for use in unit tests.
 * The initial value just contains classpath services.
 */
public class MockLookup extends ProxyLookup
  {
    private static MockLookup DEFAULT;
    private static boolean making = false;

    private static Set<Object> BANNED_OBJECTS = new HashSet<Object>();

    private static Lookup METAINF_LOOKUP = Lookup.EMPTY;
    
    private static final Lookup FILTERED_METAINF_LOOKUP = new Lookup()
      {
        @Override
        public <T> T lookup (final @Nonnull Class<T> clazz)
          {
            final Collection<? extends T> allInstances = lookup(new Template<T>(clazz)).allInstances();
            return (allInstances.isEmpty()) ? null : allInstances.iterator().next();
          }

        @Override
        public <T> Result<T> lookup (final @Nonnull Template<T> template)
          {
            final Result<T> result = METAINF_LOOKUP.lookup(template);
            return new Result<T>()
              {
                @Override
                public void addLookupListener (final LookupListener listener)
                  {
                    result.addLookupListener(listener);
                  }

                @Override
                public void removeLookupListener (final LookupListener listener)
                  {
                    result.removeLookupListener(listener);
                  }

                @Override
                public Collection<? extends T> allInstances()
                  {
                    final Collection<? extends T> allInstances = new ArrayList<T>(result.allInstances());
                    allInstances.removeAll(BANNED_OBJECTS);
                    return allInstances;
                  }
              };
          }
      };

    static
      {
        making = true;

        try
          {
            System.setProperty("org.openide.util.Lookup", MockLookup.class.getName());

            if (Lookup.getDefault().getClass() != MockLookup.class)
              {
                // Someone else initialized lookup first. Try to force our way.
                Field defaultLookup = Lookup.class.getDeclaredField("defaultLookup");
                defaultLookup.setAccessible(true);
                defaultLookup.set(null, null);
              }
            
            assertEquals(MockLookup.class, Lookup.getDefault().getClass());
          }
        catch (Exception x)
          {
            throw new ExceptionInInitializerError(x);
          }
        finally
          {
            making = false;
          }
      }

    /** Do not call this directly! */
    public MockLookup()
      {
        assertTrue(making);
        assertNull(DEFAULT);
        DEFAULT = this;
        setInstances();
      }

    /**
     * Just ensures that this lookup is default lookup, but does not actually change its content.
     * Useful mainly if you have some test utility method which calls foreign code which might use default lookup,
     * and you want to ensure that any users of mock lookup will see the correct default lookup right away,
     * even if they have not yet called {@link #setLookup} or {@link #setInstances}.
     */
    public static void init() 
      {
      }

    public static void reset()
      {
        BANNED_OBJECTS.clear();
        setInstances();
      }

    /**
     * Sets the global default lookup with zero or more delegate lookups.
     * Caution: if you don't include Lookups.metaInfServices, you may have trouble,
     * e.g. {@link #makeScratchDir} will not work.
     * Most of the time you should use {@link #setInstances} instead.
     */
    public static void setLookup (final @Nonnull Lookup... lookups)
      {
        DEFAULT.setLookups(lookups);
      }

    /**
     * Sets the global default lookup with some fixed instances.
     * Will also include (at a lower priority) a {@link ClassLoader},
     * and services found from <code>META-INF/services/*</code> in the classpath.
     */
    public static void setInstances (final @Nonnull Object... instances)
      {
        final ClassLoader l = MockLookup.class.getClassLoader();
        METAINF_LOOKUP = Lookups.metaInfServices(l);
        setLookup(Lookups.fixed(instances), FILTERED_METAINF_LOOKUP, Lookups.singleton(l));
      }

    public static void addBannedObject (final @Nonnull Object bannedObject)
      {
        BANNED_OBJECTS.add(bannedObject);
      }
  }